/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "frame.h"
#include "display/symbol.h"
#include "rpg/world/map.h"
#include "game.h"

#include "ui/textbox.h"

char_t FRAME_BUFFER[FRAME_HEIGHT * FRAME_WIDTH];
uint8_t FRAME_COLOR[FRAME_HEIGHT * FRAME_WIDTH];

void frameInit() {
  memset(FRAME_BUFFER, ' ', FRAME_HEIGHT * FRAME_WIDTH);
  
  // Draw UI top and bottom borders
  for(uint16_t x = 0; x < FRAME_WIDTH; x++) {
    char_t c = x == 0 || x == FRAME_WIDTH-1 ? '+' : '-';
    int32_t i = (FRAME_HEIGHT - 1) * FRAME_WIDTH + x;
    FRAME_BUFFER[i] = c;
    FRAME_COLOR[i] = COLOR_WHITE;

    i = (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT) * FRAME_WIDTH + x;
    FRAME_BUFFER[i] = c;
    FRAME_COLOR[i] = COLOR_WHITE;
  }
  
  for(uint16_t y = FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT + 1; y < FRAME_HEIGHT - 1; y++) {
    int32_t i = y * FRAME_WIDTH;
    FRAME_BUFFER[i] = '|';
    FRAME_COLOR[i] = COLOR_WHITE;

    i = y * FRAME_WIDTH + FRAME_WIDTH - 1;
    FRAME_BUFFER[i] = '|';
    FRAME_COLOR[i] = COLOR_WHITE;
  }
}

void frameUpdate() {
  // Draw buffer
  entity_t *ent;
  tile_t tile;
  uint32_t i;

  map_t *map = GAME.currentMap;
  if(map == NULL) return;

  // Draw the map area.
  i = 0;
  for(uint16_t y = 0; y < FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT; y++) {
    for(uint16_t x = 0; x < FRAME_WIDTH; x++) {
      if(x >= map->width || y >= map->height) {
        FRAME_COLOR[i] = COLOR_BLACK;
        FRAME_BUFFER[i++] = ' ';
        continue;
      }

      // Entity?
      ent = mapEntityGetByPosition(map, x, y);
      if(ent != NULL) {
        FRAME_COLOR[i] = symbolGetColorByEntity(ent);
        FRAME_BUFFER[i++] = symbolGetCharByEntity(ent);
        continue;
      }

      // Tile?
      tile = mapTileGetByPosition(map, x, y, 0);
      FRAME_COLOR[i] = symbolGetColorByTile(tile);
      FRAME_BUFFER[i++] = symbolGetCharByTile(tile);
    }
  }

  // UI Area.
  if(textboxIsOpen()) {
    const char_t *text = TEXTBOX.text;
    for(uint16_t x = 1; x < FRAME_WIDTH - 1; x++) {
      if(*text == '\0') break;
      FRAME_BUFFER[(FRAME_HEIGHT - 2) * FRAME_WIDTH + x] = *text++;
      FRAME_COLOR[(FRAME_HEIGHT - 2) * FRAME_WIDTH + x] = COLOR_WHITE;
    }
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 4] = ' ';
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 3] = '>';
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 2] = ' ';
  } else {
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 4] = '-';
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 3] = '-';
    FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 2] = '-';
  }
}